use sqlx::postgres::{PgConnectOptions, PgPoolOptions};
use sqlx::{Acquire, PgPool, Postgres};

async fn take_pool_or_transaction<'c, C>(pool_or_transaction: C)
where
    C: Acquire<'c, Database = Postgres>,
{
    let mut connection = pool_or_transaction.acquire().await.unwrap();

    sqlx::query("select 1 from dual")
        .execute(&mut *connection)
        .await
        .unwrap();
}

async fn run(pool: PgPool) {
    take_pool_or_transaction(&pool).await;

    let mut t = pool.begin().await.unwrap();
    take_pool_or_transaction(&mut *t).await;
}

#[actix_web::main]
async fn main() {
    let options = PgConnectOptions::new();
    let pool = PgPoolOptions::new().connect_lazy_with(options);

    let clone = pool.clone();
    //tokio::spawn(async move {
    run(clone).await;
    //});

    sqlx::query("select 1 from dual")
        .execute(&pool)
        .await
        .unwrap();

    println!("Hello, world!");
}
